==> fast-boot-security-0.0.6.tar.gz: fast-boot-security-0.0.6/fast_boot/base.py <==
from abc import ABC
from typing import Any, Dict, List, Optional, Union

from pydantic.error_wrappers import ErrorList, ErrorWrapper, flatten_errors
from pydantic.errors import PydanticErrorMixin

from fast_boot import error_code
from fast_boot.exception import LOSError
from fast_boot.schemas import CustomBaseModel


class Base(ABC):
    def __init__(self, model=CustomBaseModel):
        self.raw_errors: List[ErrorList] = []
        self.model = model
        self._error_cache: Optional[List[Dict[str, Any]]] = None

    def errors(self) -> List[Dict[str, Any]]:
        if self._error_cache is None:
            try:
                config = self.model.__config__  # type: ignore
            except AttributeError:
                config = self.model.__pydantic_model__.__config__  # type: ignore
            self._error_cache = list(flatten_errors(self.raw_errors, config))
        return self._error_cache

    def append_error(self, loc: List[Union[str, float]], code=None, msg_template=None, error: PydanticErrorMixin = None, **kwargs) -> None:
        """
        mặc định sử dụng LOSError phải có code vs msg_template tương ứng
        :param loc: vị trí của field lỗi
        :param code: error_code
        :param msg_template:
        :param error: sử dụng error có sẵn trong package errors của pydantic
        :return:
        """
        # assert hasattr(message, error.), "Không tìm thấy msg error"
        assert (code or error), "Required code or error"
        if code:
            if msg_template is None:
                msg_template = error_code.msg_templates.get(code)
            assert msg_template, f"Required msg_template for code: {code}"
            self.raw_errors.append(ErrorWrapper(exc=LOSError(code=code, msg_template=msg_template, **kwargs), loc=tuple(loc)))
        elif error:
            self.raw_errors.append(ErrorWrapper(exc=error, loc=tuple(loc)))

    def has_error(self) -> bool:
        return bool(self.raw_errors)

    def count_errors(self) -> int:
        return len(self.raw_errors)

    def nested(
            self,
            objects: Union[dict, list],
            map_with_key: str,
            children_fields: dict,
            children_list: list = None,
            key_child_map_parent=None
    ):
        """
        thay kiểu dũ liệu trong childfields sẽ ra data như mong muốn
        - children_fields={"detail": {'t1', 't2'}}: data child la dict
        - children_fields={"detail": ['t1', 't2']}: data child la list
        - Có thể nest con vào cha theo trường hợp trên:
            + tách key mapping
        re = ctr.nested(objects=data, map_with_key='the_luong_id', children_fields={"detail": ['t1', 't2']})

        re = ctr.nested(objects=re, map_with_key='id', children_fields={"detail": ['the_luong_id', 'the_luong_name', 'detail']})

        re = ctr.nested(
            objects=NEST_PARENT_FD,
            map_with_key='id',
            children_fields={"detail": ['the_luong_id', 'the_luong_name', 'detail']},
            children_list=NEST_CHILDREN_FD
        )
        """
        objects_cp = objects.copy()

        if isinstance(objects, dict):
            object_list = [objects_cp]
        else:
            object_list = objects_cp

        if not isinstance(children_fields, dict):
            raise Exception('fields type is dict')

        if len(children_fields) < 1:
            return objects_cp

        if not objects_cp:
            return objects_cp

        for key in children_fields:
            assert isinstance(children_fields[key], (list, set)), 'children is type list'
            assert len(children_fields[key]) > 0, 'children is not null'
            # assert children_fields[key][0][-2:] == "id", "First field of child  must be primary key ID"

        if children_list:
            data_result = self.__nest_child_to_parent(
                parent_list=object_list,
                map_with_key=map_with_key,
                children_fields=children_fields,
                children_list=children_list,
                key_child_map_parent=key_child_map_parent
            )
        else:
            data_result = self.__nest_me(
                objects=object_list,
                map_with_key=map_with_key,
                fields=children_fields
            )
        if isinstance(objects, dict):
            return data_result[0] if data_result else {}
        else:
            return data_result

    def __nest_me(
            self,
            objects: list,
            map_with_key: str,
            fields: dict
    ):

        all_key_child = []
        for key_child, value_child in fields.items():
            all_key_child += list(value_child)

        nest_level_data = list(
            map(
                lambda x: self.__nest_level(data_item=x, all_key_child=all_key_child, fields=fields),
                objects
            )
        )

        key_in_parent = set()
        data_parent = dict()

        for temp in list(nest_level_data):
            if temp[map_with_key] not in key_in_parent:
                key_in_parent.add(temp[map_with_key])
                data_parent.update({
                    temp[map_with_key]: temp
                })
            else:
                for key_field, value_field in fields.items():
                    if not temp[key_field]:
                        continue

                    if isinstance(value_field, list):
                        if temp[key_field][0] not in data_parent[temp[map_with_key]][key_field]:
                            data_parent[temp[map_with_key]][key_field].append(temp[key_field][0])
        return list(data_parent.values())

    def __nest_child_to_parent(self, parent_list, map_with_key: str,
                               children_fields: dict, children_list: list = None,
                               key_child_map_parent=None):

        all_key_child = []
        for key_child, value_child in children_fields.items():
            all_key_child += list(value_child)

        nest_level_data = list(
            map(
                lambda x: self.__nest_level(data_item=x, all_key_child=all_key_child, fields=children_fields),
                children_list
            )
        )

        for parent in parent_list:
            for child in nest_level_data:
                if key_child_map_parent:
                    if parent[map_with_key] == child[key_child_map_parent]:
                        self.__nest_type(parent=parent, child=child, children_fields=children_fields)
                else:
                    if parent[map_with_key] == child[map_with_key]:
                        self.__nest_type(parent=parent, child=child, children_fields=children_fields)
        return parent_list

    @staticmethod
    def __nest_type(parent, child, children_fields):
        for key_field, value_field in children_fields.items():
            if not child[key_field]:
                continue
            if isinstance(value_field, list):
                if key_field not in parent:
                    parent.update({
                        key_field: [child[key_field][0]]
                    })
                elif child[key_field][0] not in parent[key_field]:
                    parent[key_field].append(child[key_field][0])
            else:
                parent.update({
                    key_field: child[key_field][0]
                })

    @staticmethod
    def __nest_level(data_item: dict, all_key_child: list, fields: dict):
        child_temp = {}
        parent_temp = {}
        for key_temp, value_temp in data_item.items():
            if key_temp in all_key_child:
                for key_field, value_field in fields.items():
                    if key_temp in value_field:
                        if key_field not in child_temp:
                            child_temp.update({
                                key_field: {}
                            })
                        child_temp[key_field].update({
                            key_temp: value_temp
                        })
            else:
                parent_temp.update({
                    key_temp: value_temp
                })

        for key_field, value_field in fields.items():
            if isinstance(value_field, list):
                parent_temp.update({
                    key_field: [child_temp[key_field]]
                })
            else:
                parent_temp.update({
                    key_field: child_temp[key_field]
                })

        return parent_temp

==> fast-boot-security-0.0.12.tar.gz: fast-boot-security-0.0.12/fast_boot/base.py <==
from abc import ABC
from typing import Any, Dict, List, Optional, Union

from pydantic.error_wrappers import ErrorList, ErrorWrapper, flatten_errors
from pydantic.errors import PydanticErrorMixin

from fast_boot import error_code
from fast_boot.exception import LOSError
from fast_boot.schemas import Schema


class Base(ABC):
    def __init__(self, model=Schema):
        self.raw_errors: List[ErrorList] = []
        self.model = model
        self._error_cache: Optional[List[Dict[str, Any]]] = None

    def errors(self) -> List[Dict[str, Any]]:
        if self._error_cache is None:
            try:
                config = self.model.__config__  # type: ignore
            except AttributeError:
                config = self.model.__pydantic_model__.__config__  # type: ignore
            self._error_cache = list(flatten_errors(self.raw_errors, config))
        return self._error_cache

    def append_error(self, loc: List[Union[str, float]], code=None, msg_template=None, error: PydanticErrorMixin = None, **kwargs) -> None:
        """
        mặc định sử dụng LOSError phải có code vs msg_template tương ứng
        :param loc: vị trí của field lỗi
        :param code: error_code
        :param msg_template:
        :param error: sử dụng error có sẵn trong package errors của pydantic
        :return:
        """
        # assert hasattr(message, error.), "Không tìm thấy msg error"
        assert (code or error), "Required code or error"
        if code:
            if msg_template is None:
                msg_template = error_code.msg_templates.get(code)
            assert msg_template, f"Required msg_template for code: {code}"
            self.raw_errors.append(ErrorWrapper(exc=LOSError(code=code, msg_template=msg_template, **kwargs), loc=tuple(loc)))
        elif error:
            self.raw_errors.append(ErrorWrapper(exc=error, loc=tuple(loc)))

    def has_error(self) -> bool:
        return bool(self.raw_errors)

    def count_errors(self) -> int:
        return len(self.raw_errors)
